use Renard::Incunabula::Common::Setup;
package Renard::Curie;
# ABSTRACT: A document reader written with GTK+.
$Renard::Curie::VERSION = '0.005';
1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Renard::Curie - A document reader written with GTK+.

=head1 VERSION

version 0.005

=head1 SEE ALSO

L<Repository information|http://project-renard.github.io/doc/development/repo/curie/>

=head1 AUTHOR

Project Renard

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by Project Renard.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
