use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
AccelMap
Adapter
Any
App
BUILDARGS
Bach
Chirag
Component
Container
Curie
Document
DocumentModel
DocumentPropertiesWindow
DragAndDrop
Error
ExceptionHandler
FOREIGNBUILDARGS
FileChooser
ForDocument
Grid
GridView
Gtk
GtkPackType
HasParentMainWindow
Helen
HighlightCurrentSentence
JacquardCanvas
KeyBindings
LaxPageNumber
Log
LogWindow
MainWindow
MenuBar
Model
Mouse
MouseCursor
MousePageTooltip
MouseScrollBindings
MouseSelection
MuPDF
Mughal
NavigationButtons
Outline
PDF
PNG
PageActor
PageCount
PageDrawingArea
PageEntry
PageLabel
PageNumber
Pageable
Percentage
Project
Renard
RenderableDocumentModel
RenderablePageModel
Role
SVG
Scenegraph
Scroll
SizeRequest
Stanislav
Subview
SubviewPageable
TTS
TTSWindow
UI
View
ViewManager
ViewModel
ViewOptions
Yotov
Zakariyya
Zoom
ZoomLevel
bin
chiragghanshani
curie
decrement
haydeez
initialises
keybindings
lib
mutool
revealer
scrollbar
submenu
subview
subwidgets
tooltip
toplevel
viewport
yotov
zaki
